#include <string.h>
#include <stdlib.h>
#include <iostream.h>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/framework/XMLFormatter.hpp>

#define RES_NAME "Xerces"
#define PHP_XERCES_OPTION_DO_NAMESPACES 1
#define PHP_XERCES_OPTION_DO_SCHEMA 2
#define PHP_XERCES_OPTION_SCHEMA_FULL_CHECKING 3
#define PHP_XERCES_OPTION_VAL_SCHEME 4
#define PHP_XERCES_OPTION_EXTERNAL_SCHEMA_LOCATION 5

XERCES_CPP_NAMESPACE_USE

XERCES_CPP_NAMESPACE_BEGIN
class AttributeList;
XERCES_CPP_NAMESPACE_END

class res;

class StrX
{
  public :
  StrX(const XMLCh* const toTranscode)
  {
    fLocalForm=XMLString::transcode(toTranscode);
  }

  ~StrX()
  {
    XMLString::release(&fLocalForm);
  }

  const char* localForm() const
  {
    return fLocalForm;
  }

  private :
  char* fLocalForm;
};

inline ostream& operator<<(ostream& target, const StrX& toDump)
{
  target << toDump.localForm();
  return target;
}

class MemParseHandlers : public HandlerBase, public XMLFormatTarget
{
  public:
  MemParseHandlers(res *parser,char *encod);
  ~MemParseHandlers();

  void writeChars(const XMLByte* const toWrite,
    const unsigned int count,
    XMLFormatter* const formatter)
  {
    efree(membuf);
    membuf=(char *)emalloc(count+1);
    strncpy(membuf,(const char*)toWrite,count);
    membuf[count]=0;
  }

  void startElement(const XMLCh* const name, AttributeList& attributes);
  void endElement(const XMLCh* const name);
  void characters(const XMLCh* const chars, const unsigned int length);
  void ignorableWhitespace(const XMLCh* const chars, const unsigned int length);
  void resetDocument();

  void warning(const SAXParseException& exception);
  void error(const SAXParseException& exception);
  void fatalError(const SAXParseException& exception);

  private:
  res *parser;
  XMLFormatter    fFormatter;
  char *membuf;
};

class res
{
  public:
  res(char *encod);
  ~res();
  void set_object(zval *object);
  void parse(zval *string);
  void set_handler(zval **handler, zval **data);
  zval *call_handler(res *parser, zval *handler, int argc, zval **argv);

  zval *se_handler, *ee_handler, *c_handler, *df_handler;
  zval *err_handler, *ferr_handler, *war_handler;
  int index;
  bool doNamespaces;
  bool doSchema;
  char externalSchemaLocation[0x100];
  bool schemaFullChecking;
  SAXParser::ValSchemes valScheme;
  
  private:
  SAXParser *parser;
  MemParseHandlers *handler;
  zval *object;
};
